package de.intevation.flys.client.client.ui;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;

import com.smartgwt.client.widgets.Canvas;
import com.smartgwt.client.widgets.Img;
import com.smartgwt.client.widgets.events.ClickEvent;
import com.smartgwt.client.widgets.events.ClickHandler;
import com.smartgwt.client.widgets.Button;

import de.intevation.flys.client.client.FLYSConstants;
import de.intevation.flys.client.client.event.HasStepBackHandlers;
import de.intevation.flys.client.client.event.HasStepForwardHandlers;
import de.intevation.flys.client.client.event.StepBackEvent;
import de.intevation.flys.client.client.event.StepBackHandler;
import de.intevation.flys.client.client.event.StepForwardEvent;
import de.intevation.flys.client.client.event.StepForwardHandler;
import de.intevation.flys.client.shared.model.Data;
import de.intevation.flys.client.shared.model.DataList;


/**
 * An abstract UIProvider that provides some basic methods.
 *
 * @author <a href="mailto:ingo.weinzierl@intevation.de">Ingo Weinzierl</a>
 */
public abstract class AbstractUIProvider
implements   UIProvider, HasStepForwardHandlers, ClickHandler,
             HasStepBackHandlers
{
    /** The message class that provides i18n strings.*/
    protected FLYSConstants MSG = GWT.create(FLYSConstants.class);


    /** The StepForwardHandlers.*/
    protected List<StepForwardHandler> forwardHandlers;

    /** The StepForwardHandlers.*/
    protected List<StepBackHandler> backHandlers;

    /** The container that is used to position helper widgets.*/
    protected Canvas helperContainer;


    /**
     * Creates a new UIProvider instance of this class.
     */
    public AbstractUIProvider() {
        forwardHandlers = new ArrayList<StepForwardHandler>();
        backHandlers    = new ArrayList<StepBackHandler>();
    }


    /**
     * Appends a StepBackHandler that wants to listen to StepBackEvents.
     *
     * @param handler A new StepBackHandler.
     */
    public void addStepBackHandler(StepBackHandler handler) {
        if (handler != null) {
            backHandlers.add(handler);
        }
    }


    /**
     * Appends a StepForwardHandler that wants to listen to StepForwardEvents.
     *
     * @param handler A new StepForwardHandler.
     */
    public void addStepForwardHandler(StepForwardHandler handler) {
        if (handler != null) {
            forwardHandlers.add(handler);
        }
    }


    /**
     * This method is called after the user has clicked one of the buttons to
     * step back to a previous state.
     *
     * @param e The StepBackEvent.
     */
    protected void fireStepBackEvent(StepBackEvent e) {
        GWT.log("AbstractUIProvider - fireStepBackEvent() handlers: " + backHandlers.size());
        for (StepBackHandler handler: backHandlers) {
            handler.onStepBack(e);
        }
    }


    /**
     * This method is called after the user has clicked on the 'next' button to
     * step to the next state.
     *
     * @param e The StepForwardEvent.
     */
    protected void fireStepForwardEvent(StepForwardEvent e) {
        GWT.log("AbstractUIProvider - fireStepForwardEvent() handlers: " + forwardHandlers.size());
        for (StepForwardHandler handler: forwardHandlers) {
            handler.onStepForward(e);
        }
    }


    /**
     * This method is used to listen to click events on the 'next' button. The
     * fireStepForwardEvent() method is called here.
     *
     * @param e The click event.
     */
    public void onClick(ClickEvent e) {
        Data[] data = getData();

        fireStepForwardEvent(new StepForwardEvent(data));
    }


    /**
     * Creates the 'next' button to step forward to the next state.
     *
     * @return the 'next' button.
     */
    protected Canvas getNextButton() {
        Button next = new Button(MSG.buttonNext());
        next.addClickHandler(this);

        return next;
    }


    /**
     * Creates the 'back' button to step back to a previous state.
     *
     * @param targetState The identifier of the target state.
     *
     * @return the 'back' button.
     */
    protected Canvas getBackButton(final String targetState) {
        String url = GWT.getHostPageBaseURL() + MSG.imageBack();
        Img back   = new Img(url, 16, 16);

        back.addClickHandler(new ClickHandler() {
            public void onClick(ClickEvent event) {
                fireStepBackEvent(new StepBackEvent(targetState));
            }
        });

        return back;
    }


    /**
     * This method injects a container that is used to position helper widgets.
     *
     * @param helperContainer A container that is used to position helper
     * widgets.
     */
    public void setContainer(Canvas helperContainer) {
        this.helperContainer = helperContainer;
    }


    /**
     * This method needs to be implemented by concrete subclasses. It should
     * create a new Canvas object with a representation of <i>data</i>.
     *
     * @param data The data that should be displayed.
     *
     * @return a Canvas object that displays <i>data</i>.
     */
    public abstract Canvas create(DataList data);


    /**
     * This method needs to be implemented by concrete subclasses. It should
     * return the selected data.
     *
     * @return the selected data.
     */
    protected abstract Data[] getData();
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
